// Matthias Hollnder 2013
#version 430 core

#pragma MX_PORT_SURFACE(o_FragColor,"Color-Target","Color");
#pragma MX_PORT_SURFACE(o_FragNormal,"Normal-Target","Normal");
#pragma MX_PORT_SURFACE(o_FragTexCoord,"TexCoord-Target","TexCoord");

#pragma MX_PORT_SURFACE(o_WorldSpace,"WorldSpace-Target","WorldSpace");


//////////////////////////////////////////////////////////////////////////
// Input textures
//////////////////////////////////////////////////////////////////////////
layout( binding = 0 ) uniform sampler2D sampTexture;

layout( location = 3 ) uniform vec3 camPos;

//////////////////////////////////////////////////////////////////////////
// Incoming data
//////////////////////////////////////////////////////////////////////////
in block
{
	vec3 Normal;
	vec3 WorldSpace;
	vec2 TexCoord;
	float near;
	float far;
} In;

//////////////////////////////////////////////////////////////////////////
// output variables
//////////////////////////////////////////////////////////////////////////

layout( location = 0 ) out vec4 o_FragColor;
layout( location = 1 ) out vec3 o_FragNormal;
layout( location = 2 ) out vec3 o_FragTexCoord;
layout( location = 3 ) out vec3 o_WorldSpace;

//////////////////////////////////////////////////////////////////////////
// Fragment Shader
//////////////////////////////////////////////////////////////////////////
void main()
{
	vec4 vTexColor = texture( sampTexture, In.TexCoord ).xyzw;
	if ( vTexColor.w < 0.1 ) discard;
	
	float interp = (gl_FragCoord.z/gl_FragCoord.w - In.near) / In.far;
	interp = interp > 1.0 ? 1.0 : interp < 0.0 ? 0.0 : interp;
	
	vec3 r = vec3(1,0,0);
	vec3 w = vec3(1);
	vec3 b = vec3(0,0,1);

	vec3 color;
	if (interp < 0.5)
		color = 2.0*interp*w + (1.0-2.0*interp)*r;
	else
		color = (2.0-2.0*interp)*w + (2.0*interp-1.0)*b;


	vec3 lightDir = vec3(1) - In.WorldSpace;
	float d = length( lightDir );
	lightDir = normalize(lightDir);
 
	//Intensity of the diffuse light. Saturate to keep within the 0-1 range.
	float NdotL = dot( normalize( In.Normal ), lightDir );
	float intensity = clamp( NdotL, 0.0, 1.0 );
 
	// Calculate the diffuse light factoring in light color, power and the attenuation
	vec3 diffuse = intensity * color / d;
 
	//Calculate the half vector between the light vector and the view vector.
	//This is faster than calculating the actual reflective vector.
	vec3 viewDir = camPos - In.WorldSpace;
	vec3 H = normalize( lightDir + viewDir );
 
	//Intensity of the specular light
	float NdotH = dot( normalize( In.Normal ), H );
	intensity = pow( clamp( NdotH, 0.0, 1.0 ), 1.0 );
 
	//Sum up the specular light factoring
	vec3 specular = intensity * color / d;

	o_FragColor = 0.1*vTexColor+0.9*vec4(diffuse+specular, 1.0);
	o_FragNormal = normalize( In.Normal );
	o_FragTexCoord = vec3( In.TexCoord, 0.0 );
	o_WorldSpace = In.WorldSpace;
}
