// Matthias Hollnder 2013
#version 430 core

#pragma MX_PORT_UNIFORM(InDepth,"Depth In");

//////////////////////////////////////////////////////////////////////////
// Input variables
//////////////////////////////////////////////////////////////////////////
layout( binding = 0 ) uniform sampler2D InDepth;

layout( location = 0 ) uniform vec2 u_vInvFocalLen;
layout( location = 1 ) uniform float u_fNear;
layout( location = 2 ) uniform float u_fFar;

//////////////////////////////////////////////////////////////////////////
// Incoming data
//////////////////////////////////////////////////////////////////////////
in block
{
	vec2 TexCoord;
} In;

//////////////////////////////////////////////////////////////////////////
// output variables
//////////////////////////////////////////////////////////////////////////
layout( location = 0 ) out vec4 o_FragData;
//~ layout( binding = 2 ) out vec4 o_FragData;

float GetZValue( ivec2 ivSampleCoord )
{
	float fDepth = texelFetch( InDepth, ivSampleCoord, 0 ).x;
	return ( -u_fNear * u_fFar ) / ( u_fFar - ( fDepth * ( u_fFar - u_fNear ) ) );
}

// For reconstructing the view space position
// using the depth buffer and the fragment coordinate
vec3 GetPosition( ivec2 ivSampleCoord )
{
	float fDepth = GetZValue( ivSampleCoord );
	vec2 vUV = vec2( ivSampleCoord ) / vec2( textureSize( InDepth, 0 ) );
	
	// range scaling
	vUV = ( vUV * vec2( 2.0, -2.0 ) - vec2( 1.0, -1.0 ) );
	
	// scale by depth and use inverse focal length
	return vec3( vUV * vec2( u_vInvFocalLen.x, u_vInvFocalLen.y ) * fDepth, fDepth );
}

//////////////////////////////////////////////////////////////////////////
// Fragment Shader
//////////////////////////////////////////////////////////////////////////
void main()
{
	vec3 vPosition = GetPosition( ivec2( gl_FragCoord.xy ) );
	vec3 vOutput = normalize( cross( dFdx( vPosition ), dFdy( vPosition ) ) );
	o_FragData = vec4( vOutput, 1.0 );
}
