// Matthias Hollnder 2013
#version 430 core

#pragma MX_PORT_UNIFORM(sampDepth,"Depth In");
#pragma MX_PORT_UNIFORM(sampNormals,"Normals In");

//#pragma MX_PORT_UNIFORM(sample_positions,"Sample Positions");
//#pragma MX_PORT_UNIFORM(sample_directions,"Sample Directions");

#pragma MX_PORT_SURFACE(o_FragData,"Target Surface","AO");

//////////////////////////////////////////////////////////////////////////
// Input variables
//////////////////////////////////////////////////////////////////////////
layout( binding = 0 ) uniform sampler2D sampDepth;
layout( binding = 1 ) uniform sampler2D sampNormals;

// SSAO Settings
uniform float ssao_radius = 5.0;
uniform uint point_count = 8;
uniform bool randomize_points = true;

//////////////////////////////////////////////////////////////////////////
// Incoming data
//////////////////////////////////////////////////////////////////////////
in block
{
	vec3 Eye;
	vec2 TexCoord;
} In;


//uniform vec3 sample_positions[ 256 ];
//uniform vec3 sample_directions[ 256 ];
layout( binding = 0, std140 ) uniform sample_points
{
    vec4 pos[ 256 ];
    vec4 random_vectors[ 256 ];
} points;

//////////////////////////////////////////////////////////////////////////
// output variables
//////////////////////////////////////////////////////////////////////////
layout( location = 0, index = 0 ) out vec4 o_FragData;

//////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////
float ComputeAO( vec2 uv )
{
	// Normal and depth
    vec3 N       = textureLod( sampNormals, uv, 0 ).xyz;
    float fDepth = textureLod( sampDepth, uv, 0 ).x;
	
    // temp variables
    int i;
    int j;
    int k;

    float occ   = 0.0;
    float total = 0.0;

    // k is a pseudo-random number generated from fragment coordinate
    // and depth
    k = ( int( gl_FragCoord.x * 7123.2315 + 125.232 ) *
          int( gl_FragCoord.y * 3137.1519 + 234.8 ) ) ^
          int( fDepth );

    // Pull one of the random vectors
	vec4 v = points.random_vectors[ k & 255 ];

    // r is our 'radius randomizer'
    float r = ( v.r + 3.0 ) * 0.1;
    if ( !randomize_points ) {
        r = 0.5;
	}
	
	return gl_FragCoord.x;

    // For each random point (or direction)...
    for( i = 0; i < point_count; ++i ) {
        // Get direction
        //vec3 dir = points.pos[ i ].xyz;
		vec3 dir = points.pos[ i ].xyz;
		

        // Put it into the correct hemisphere
        if( dot( N, dir ) < 0.0 ) {
            dir = -dir;
		}

        // f is the distance we've stepped in this direction
        // z is the interpolated depth
        float f = 0.0;
        float z = fDepth;

        // We're going to take 4 steps - we could make this
        // configurable
        total += 4.0;

        for( j=0; j<4; ++j ) {
            // Step in the right direction
            f += r;
            // Step _towards_ viewer reduces z
            z -= dir.z * f;

            // Read depth from current fragment
            float their_depth = textureLod( sampDepth, ( uv + dir.xy * f * ssao_radius ), 0 ).x;

            // Calculate a weighting (d) for this fragment's
            // contribution to occlusion
            float d = abs( their_depth - fDepth );
            d *= d;

            // If we're obscured, accumulate occlusion
            if ( ( z - their_depth) > 0.0 ) {
                occ += 4.0 / ( 1.0 + d );
            }
        }
    }

    // Calculate occlusion amount
    //float ao_amount = vec4(1.0 - occ / total);

	return 1.0 - occ / total;
}

//////////////////////////////////////////////////////////////////////////
// Fragment Shader
//////////////////////////////////////////////////////////////////////////
void main()
{
	float fAO = ComputeAO( In.TexCoord );
	
	//fAO = textureLod( sampDepth, In.TexCoord, 0 ).x;
	//fAO *= 100.0;
	//fAO = ( fAO < 0.251 ? 1.0 : 0.0 ); 
	o_FragData = vec4( fAO, fAO, fAO, 1.0 );
	
	//o_FragData = points.pos[ 2 ];
}
