// Matthias Hollnder 2013
#version 430 core

#pragma MX_PORT_SURFACE(o_FragData,"Color-Target","Color");

//////////////////////////////////////////////////////////////////////////
// Input textures
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
// Incoming data
//////////////////////////////////////////////////////////////////////////
in block
{
	vec3 			WorldSpace;
	vec3 			Normal;
} In;

struct Light
{
	vec4			m_vPosition;
	vec4			m_vColor;
	vec3			m_vAttenuation;
	float			m_fSpotCutoff;
	vec3			m_vSpotDirection;
	int				m_iLightType;
};

struct Material
{
	vec4 			m_vEmission;
	vec4 			m_vAmbient;
	vec4 			m_vDiffuse;
	vec4 			m_vSpecular;
	float 			m_fShininess;
};

uniform Material	u_Material;
uniform Light		LightSources[ 8 ];
uniform int			NumberOfLights = 1;
uniform vec3		CameraPosition;

uniform float 		JingTangScalar;

//////////////////////////////////////////////////////////////////////////
// output variables
//////////////////////////////////////////////////////////////////////////

layout( location = 0, index = 0 ) out vec4 o_FragData;


vec4 ComputeColor()
{
	vec3 vNormal = normalize( In.Normal );
	vec3 vWorldSpace = In.WorldSpace;
	vec3 vEye = vWorldSpace - CameraPosition;
	vec4 vColor = vec4( 0.0 );

	for( int i=0; i<NumberOfLights; ++i ) {
		vec3 vLightVec = LightSources[i].m_vPosition.xyz - vWorldSpace;
		float fDistanceToLight = length( vLightVec );
		vec3 vLightDir = ( vLightVec / fDistanceToLight );

		// Diffuse
		float fLambert = max( dot( vLightDir, vNormal ), 0.0 );
		vec4 vDiffuse = fLambert * u_Material.m_vDiffuse * LightSources[i].m_vColor;

		// Specular
		//vec3 vReflected = reflect( -vLightDir, vNormal );
		vec3 vReflected = normalize( -reflect( vLightDir, vNormal ) );
		float fTemp = dot( vReflected, normalize( vWorldSpace ) );
		float fSpecularIntensity = pow( max( dot( vReflected, normalize( -vEye ) ), 0.0 ), u_Material.m_fShininess );
		vec4 vSpecular = fSpecularIntensity * ( LightSources[i].m_vColor * u_Material.m_vSpecular );

		
		float fAttenuation = 1.0 / ( LightSources[i].m_vAttenuation.x + 
									 LightSources[i].m_vAttenuation.y * fDistanceToLight +
									 LightSources[i].m_vAttenuation.z * fDistanceToLight * fDistanceToLight );
		
		vColor += fAttenuation * ( vDiffuse + vSpecular );
	}
	
	vColor.w = 1.0;

	// for debugging:
	//vColor.xyz = u_Material.m_vDiffuse.xyz;
	//vColor.xyz = vNormal;
	//vColor.xyz = ( dot( vNormal, vNormal ) > 0.5 ) ? vColor.xyz : vMatDiffuse.xyz;
	
	return vColor;
}


//////////////////////////////////////////////////////////////////////////
// Fragment Shader
//////////////////////////////////////////////////////////////////////////
void main()
{
	o_FragData = ComputeColor();
}
