// Timothy Kol & Jingtang Liao 2014
#version 430 core

// Define ports
#pragma MX_PORT_SURFACE(o_FragColor,"Color Target", "Color");
#pragma MX_PORT_SURFACE(o_FragNormal,"Normal Target", "Normals");
#pragma MX_PORT_SURFACE(o_WorldSpace,"WorldSpace Target", "WorldSpace");


// Input
in block {
	vec3 Normal;
	vec3 WorldSpace;
	vec2 TexCoord;
} In;

// Output
layout( location = 0 ) out vec4 o_FragColor;
layout( location = 1 ) out vec3 o_FragNormal;
layout( location = 2 ) out vec3 o_WorldSpace;

// Color sampler
layout( binding = 0 ) uniform sampler2D sampTexture;

// Camera position
layout ( location = 4 ) uniform vec3 C;

// The rendering mode: 1 is canonical buildings (first pass), 2 is canonical buildings (second pass), 3 is terrain or non-canonical buildings
layout ( location = 7 ) uniform int M;

// The id
layout ( location = 8 ) uniform int id;

// The texture containing the correct id's
layout ( location = 9 ) uniform sampler2D idTexture;

// Fragment shader
void main() {

	// Get correct id from texture
	vec3 idCorrect = vec3(0);//texture(idTexture, gl_FragCoord.xz/gl_FragCoord.w).rgb;
	
	// Convert current id to vec3
	float r = float( id % 256 );
	float g = float( (id/256) % 256 );
	float b = float( (id/65536) % 256 );
	vec3 idVec = vec3(r,g,b)/255.0f;
	
	// If we are in mode 2 and the current id does not equal the correct id, discard pixel
	if (M == 2 && idCorrect != idVec)
		;//discard;

	// Sample color
	vec4 color = texture( sampTexture, In.TexCoord );

	// Discard if alpha < 0.1
	if (color.w < 0.1f)
		discard;

	// Write color, normal and world space
	o_FragColor = color;
	o_FragNormal = In.Normal;
	o_WorldSpace = In.WorldSpace;

	// Write id and special depth in mode 1
	if (M == 1) {
		//o_Id = idVec;
		gl_FragDepth = distance(C.xz, In.WorldSpace.xz);

	// Use normal depth in modes 2 and 3
	} else
		gl_FragDepth = gl_FragCoord.z;
}