// Timothy Kol & Jingtang Liao 2014
#version 430 core

// Define input ports
#pragma MX_PORT_ATTRIBUTE(Position,"Positions");
#pragma MX_PORT_ATTRIBUTE(Normal,"Normals");
#pragma MX_PORT_ATTRIBUTE(TexCoord,"TextureCoords");

// Input
layout( location = 0 ) in vec3 Position;
layout( location = 1 ) in vec3 Normal;
layout( location = 2 ) in vec2 TexCoord;

// Matrices
layout ( location = 0 ) uniform mat4 MVP;
layout ( location = 1 ) uniform mat4 WorldMatrix;
layout ( location = 2 ) uniform mat3 NormalMatrix;

// Canonical angle
layout ( location = 3 ) uniform float theta_c;

// Camera position
layout ( location = 4 ) uniform vec3 C;

// Bounding box top position
layout ( location = 5 ) uniform vec3 c;

// Bounding box bottom y
layout ( location = 6 ) uniform float bottom;

// The rendering mode: 1 is canonical (first pass), 2 is canonical (second pass), 3 is non-canonical
layout ( location = 7 ) uniform int M;


// Output
out block {
	vec3 Normal;
	vec3 WorldSpace;
	vec2 TexCoord;
} Out;

// Vertex shader
void main() {

	// Save position
	vec3 p0 = Position;

	// Set normal output
	Out.Normal = NormalMatrix * Normal;
	// Set world space output
	Out.WorldSpace = (WorldMatrix * vec4(p0, 1.0f)).xyz;
	// Set texture coordinate output
	Out.TexCoord = TexCoord;

	// Vector pointing from center of bounding box top side to camera
	vec3 v = C - c;
	
	// Compute elevation
	float theta = abs(  acos( min( dot( normalize(v), normalize(vec3(v.x, 0.0f, v.z)) ) , 0.99999f) )  );

	// Negative theta if camera is below bounding box top
	if (v.y < 0.0f)
		theta *= -1.0f;

	// Compute elevation offset
	float theta_delta = theta - theta_c;

	// Prevent bending forward
	theta_delta = max(theta_delta, 0.0f);

	// Vertex bending angle in z and x directions, based on the vector v
	float theta_delta_Z = normalize(v.xz).x * theta_delta;
	float theta_delta_X = -normalize(v.xz).y * theta_delta;
	
	// Precompute rotation matrices values
	float cos_theta_delta_Z = cos(theta_delta_Z);
	float sin_theta_delta_Z = sin(theta_delta_Z);
	float cos_theta_delta_X = cos(theta_delta_X);
	float sin_theta_delta_X = sin(theta_delta_X);

	// Compute rotation matrices in z and x directions
	mat3 mRotZ = mat3(
		 cos_theta_delta_Z, sin_theta_delta_Z, 0.0f,
		-sin_theta_delta_Z, cos_theta_delta_Z, 0.0f,
		0.0f, 0.0f, 1.0f
	);
	mat3 mRotX = mat3(
		1.0f, 0.0f, 0.0f,
		0.0f,  cos_theta_delta_X, sin_theta_delta_X,
		0.0f, -sin_theta_delta_X, cos_theta_delta_X
	);

	
	// Compute p_prime
	vec3 p_prime = vec3(p0.x, bottom, p0.z);

	// Rotate around point straight below vertex on bounding box bottom side
	vec3 p1 = p_prime + mRotX * mRotZ * (p0 - p_prime);

	// Constrain to old height, moving in camera direction
	vec3 p2 = p1 + v * (p0.y - p1.y) / v.y;

	// Set final position
	vec3 p = M == 3 ? p0 : p2;	

	// Save clip position
	gl_Position = MVP * vec4( p, 1.0 );
}